/*
* Distributed Computational System
* Prime Number Calculator - Client Plugin
*
* $Id: cp_primes.c,v 1.8 2001/11/30 00:33:52 kris Exp $
*/

#define COMPILING_DLL
#include "client_plugin.h"
#include "cp_precalc.h"
#include <math.h>	/* for sqrt */
#include <string.h>	/* for memcpy */
#include <stdio.h>

#ifdef _WIN32
	#include <windows.h>
#endif

int isprime( unsigned long num );

int g_plugin_id = -1;

DLLEXPORT int CP_Init( int plugin_id )
{
	g_plugin_id = plugin_id;
	return 0;
}

DLLEXPORT int CP_GetVersion()
{
	return 0;
}

DLLEXPORT int CP_AssignProblemChunk( cl_reportresult_f report, unsigned char type, client_chunk_t *chunk )
{
	/* we ignore type in our prototype */
	/* data format for this prototype plugin: [range-min(4)][range-max(4)] */

	unsigned long min, max, i;
	int n = 0;
	FILE *tmp;
	client_result_t result;

	if( chunk->len < 8 )
		return 1;

	memcpy( &min, chunk->data, 4 );
	memcpy( &max, chunk->data + 4, 4 );

	fprintf( stderr, "Working on range %d to %d\n", min, max );

	/*
		we're gonna be cheap with memory, since we don't
		know the range we're dealing with -- we'll write
		results to a temp file and then read that file in
		only at the end
	*/
	tmp = fopen( "primes.tmp", "wb+" );

	/* do all the work */
	for( i = min; i <= max; i++ ) {
		if( isprime( i ) ) {
			fwrite( &i, 4, 1, tmp );
			n++;
		}
	}

	fseek( tmp, 0, SEEK_SET );

	/*
		this allocation could be really large, in a real
		system we might want to provide a streaming
		mechanism to keep the memory usage lower
	*/
	result.len = 4 * n;
	result.data = (unsigned char *)malloc( result.len );
	fread( result.data, 4, n, tmp );
	fclose( tmp );

	(report)( g_plugin_id, &result );

	free( result.data );

	return 0;
}

int isprime( unsigned long num )
{
	int i;

	for( i = 0; (g_precalc[i] < (unsigned long)sqrt( (double)num ) + 1) && (g_precalc[i] != 0); i++ ) {
		if( num % g_precalc[i] == 0 )
			return 0;
	}
	return 1;
}
