/*
* Distributed Computational System
* Prototype Database
* This doesn't follow any of the specs for our real database
* API, I simply made this for convenience reasons while
* making the prototype; its sole purpose is keeping track of
* chunks that haven't been completed yet in case a client
* that has been assigned a problem disconnects (so we don't
* forget to eventually do the work for that problem chunk).
*
* Note that this is all problem-specific (for prime numbers)!
*
* Also note that, since this is a prototype and we don't
* care, all of this is stored in memory (since we are already
* limiting our problem size anyway).
*
* $Id: db_proto.h,v 1.3 2001/11/29 07:06:17 kris Exp $
*/

#ifndef _DB_PROTO_H
#define _DB_PROTO_H

#include <string.h>

typedef struct _old_chunk_t {
	unsigned char chunk_id[16];
	unsigned long min;
	unsigned long max;
} old_chunk_t;

typedef struct _dbp_queue_t {
	old_chunk_t chunk;
	struct _dbp_queue_t *next;
} dbp_queue_t;

dbp_queue_t *g_head = 0, *g_tail = 0;

int DBP_PopOldChunk( old_chunk_t *buf )
{
	dbp_queue_t *item;

	if( g_head == 0 )
		return -1;
	item = g_head;
	g_head = g_head->next;
	if( item == g_tail )
		g_tail = 0;
	memcpy( buf, &item->chunk, sizeof( old_chunk_t ) );
	free( item );
	return 0;
}

int DBP_PushOldChunk( old_chunk_t *buf )
{
	dbp_queue_t *item;

	item = (dbp_queue_t *)malloc( sizeof( dbp_queue_t ) );
	memcpy( &item->chunk, buf, sizeof( old_chunk_t ) );
	item->next = 0;
	if( !g_tail )
		g_tail = g_head = item;
	else {
		g_tail->next = item;
		g_tail = item;
	}
	return 0;
}

int DBP_RemoveOldChunk( unsigned char chunk_id[16] )
{
	dbp_queue_t *i, *p;

	if( g_head == 0 )
		return -1;
	if( !memcmp( g_head->chunk.chunk_id, chunk_id, 16 ) ) {
		i = g_head;
		g_head = g_head->next;
		if( i == g_tail )
			g_tail = 0;
		free( i );
		return 0;
	}
	p = g_head;
	for( i = g_head->next; i != 0; i = i->next ) {
		if( !memcmp( i->chunk.chunk_id, chunk_id, 16 ) ) {
			p->next = i->next;
			if( i == g_tail )
				g_tail = p;
			free( i );
			return 0;
		}
		p = p->next;
	}
	return -1;
}

#endif /* _DB_PROTO_H */
